@@ -1,8 +1,130 @@
 import numpy as np
 import csv
+import re
 from cobra.io import read_sbml_model
 from cobra import Reaction, Metabolite
 
+###############################################################################
+###############################################################################
+###############################################################################
+#
+# Basic functions working on strings without cobra model
+#
+###############################################################################
+###############################################################################
+###############################################################################
+
+def countCinFormula(formula):
+    '''
+    Counting C-atoms of metabolite in cobra model
+    
+    Argument:
+    formula: string, the metabolite formula from the cobra model
+    
+    Return:
+    int, number of C-atoms in formula
+    '''
+    return int(re.search(r'C(.*)H',formula).group(1)) if re.search(r'C(.*)H',formula).group(1) else int(1)
+
+###############################################################################
+###############################################################################
+###############################################################################
+#
+# Functions working on cobra model objects but without the explicit model
+#
+###############################################################################
+###############################################################################
+###############################################################################
+def MetNewFromExist(MetIn, CompSep):
+    '''
+    Generate new cobra external metabolite object from input cobra metabolite.
+
+    Parameters
+    ----------
+    MetIn : cobra metabolite object
+        Internal metabolite for which an external metabolite is generated.
+    CompSep: string='_' or '[]', separator of metabolite and compartment in id
+
+    Returns
+    -------
+    Metout : cobra metabolite object
+        External metabolite object.
+
+    '''
+    Metout = Metabolite('{}_e'.format(MetIn.id.split(CompSep[0])[0]),
+                       formula = MetIn.formula,
+                       name = MetIn.name,
+                       compartment = 'e',
+                       charge = MetIn.charge)
+    Metout.annotation = MetIn.annotation
+    return Metout
+
+def CreateTransReact(MetIn, CompSep='_'):
+    '''
+    Create transfer reaction from internal to external metabolite.
+
+    Parameters
+    ----------
+    MetIn : cobra metabolite object
+        Internal cobra metabolite object.
+    CompSep: string='_'(default) or '[]', separator of metabolite and compartment in id
+
+    Returns
+    -------
+    reaction : cobra reaction object
+        Transport reaction from cytoplasm to extracellular.
+
+    '''
+    reaction = Reaction('Trans_{}'.format(MetIn.id),
+                       name = 'Transport c<->e {}'.format(MetIn.name),
+                       lower_bound = -1000,
+                       upper_bound = 1000)
+    MetOut = MetNewFromExist(MetIn, CompSep)
+    reaction.add_metabolites({MetIn:-1.0, MetOut:1.0})
+    
+    return reaction
+###############################################################################
+###############################################################################
+###############################################################################
+#
+# Functions working on cobra model objects
+#
+###############################################################################
+###############################################################################
+###############################################################################
+
+def set_ObjGSMM(model, Product):
+    '''
+    Adding boundary flux for a desired product and setting the objective reaction to the product in the model.
+    
+    Arguments:
+    model: cobra model, iUma22
+    Product: string, model metabolite ID
+    
+    Return:
+    cobra model, updated with a demand flow for the product and optimization of the new demand reaction.
+    '''
+    model.add_boundary(model.metabolites.get_by_id(Product), type='demand')
+    model.objective = 'DM_{}'.format(Product)
+    return model
+
+def set_SubFlux(model, SubOn, SubOff, Flux):
+    '''
+    Setting the desired substrate in the model.
+    
+    Arguments:
+    model: cobra model, iOpol909
+    SubOn: string/list, model metabolite ID
+    SubOff:string, metabolite to be set to zero
+    Flux: float, desired substrate uptake rate
+    
+    Return:
+    cobra model, updated with designed (co-) substrate uptake rates.
+    '''
+    model.reactions.get_by_id(SubOff).lower_bound = 0
+    model.reactions.get_by_id('Ex_{}'.format(Sub[0])).lower_bound = -Flux
+    return model
+
 def TestSubstrate(model, EX_Sub_Act:list, EX_Sub_Off=['EX_glc__D_e'], Rate=10):
     '''
     Simulation of growth rate with defined substrates for growth and substrates not used for growth.
@@ -33,55 +155,79 @@ def TestSubstrate(model, EX_Sub_Act:list, EX_Sub_Off=['EX_glc__D_e'], Rate=10):
         medium.update({mySub: Rate for mySub in EX_Sub_Act})
         model.medium = medium
         return round(model.slim_optimize(),2)
-# =============================================================================
-# 
-# =============================================================================
-def MetNewFromExist(MetIn):
+
+
+def MetSharedRcts(model, Met1, Met2):
     '''
-    Generate new cobra external metabolite object from input cobra metabolite.
-
-    Parameters
-    ----------
-    MetIn : cobra metabolite object
-        Internal metabolite for which an external metabolite is generated.
-
-    Returns
-    -------
-    Metout : cobra metabolite object
-        External metabolite object.
-
-    '''
-    Metout = Metabolite('{}_e'.format(MetIn.id[:-2]),
-                       formula = MetIn.formula,
-                       name = MetIn.name,
-                       compartment = 'e',
-                       charge = MetIn.charge)
-    Metout.annotation = MetIn.annotation
-    return Metout
-
-def CreateTransReact(MetIn):
-    '''
-    Create transfer reaction from internal to external metabolite.
-
-    Parameters
-    ----------
-    MetIn : cobra metabolite object
-        Internal cobra metabolite object.
-
-    Returns
-    -------
-    reaction : cobra reaction object
-        Transport reaction from cytoplasm to extracellular.
-
-    '''
-    reaction = Reaction('Trans_{}'.format(MetIn.id),
-                       name = 'Transport c<->e {}'.format(MetIn.name),
-                       lower_bound = -1000,
-                       upper_bound = 1000)
-    MetOut = MetNewFromExist(MetIn)
-    reaction.add_metabolites({MetIn:-1.0, MetOut:1.0})
+    This function lists all reactions which are shared by two metabolites
     
-    return reaction
+    Parameters
+    ----------
+    model : cobra model
+        The cobra model for analysis.
+    Met1 : str
+        Metabolite id.
+    Met2 : str
+        Metabolite id.
+
+    Returns
+    -------
+    list, List of shared reactions.
+
+    '''
+    return [reaction for reaction in model.metabolites.get_by_id(Met1).reactions if reaction in model.metabolites.get_by_id(Met2).reactions]
+
+def MetFluxConnect(model, Met1, Met2, SubOff='EX_glc__D_e', ngam=False, slim=True):
+    '''
+    Test whether there exists a flux between two metabolites. The first metabolite is set as source and FBA is optimized to the second metabolite.
+
+    Parameters
+    ----------
+    model : cobra model
+        .
+        The cobra model for analysis.
+    Met1 : str
+        Metabolite id.
+    Met2 : str
+        Metabolite id.
+    SubOff: str
+        Substrate id to be set to zero
+    ngam: str
+        non growth associated maintenance to be set to zero, because checking close metabolites might not fullfill the base ATP demand in ngam
+    slim : Boolean
+        Decision whether only the value of the objective is reported via slim_optimize, or the full flux-reaction list.
+
+    Returns
+    -------
+    list, List of reactions and flux values.
+
+    '''
+    with model:
+        # medium = dict.fromkeys(model.medium, 0)
+        medium = model.medium
+        medium[SubOff] = 0
+        model.medium = medium
+        model.add_boundary(model.metabolites.get_by_id(Met1), type='sink')        
+        model.add_boundary(model.metabolites.get_by_id(Met2), type='sink')
+        if ngam:
+            model.reactions.get_by_id(ngam).lower_bound = 0
+        model.objective = 'SK_{}'.format(Met2)
+        if slim:
+            flux = model.slim_optimize()
+        else:
+            flux = model.optimize()
+            
+        return flux
+
+###############################################################################
+###############################################################################
+###############################################################################
+#
+# BIOLOG related functions
+#
+###############################################################################
+###############################################################################
+###############################################################################
 
 def CSVexport(myDict, FName='BiologGrowthTest'):
     fields = ['id','name','id_e','id_c','growth','CL_Growth']
@@ -206,64 +352,3 @@ def CheckWellinGSMM(myGrowth, myBool):
 # #        no growth in well and model lacks metabolite     
 #         return 0
 
-def MetSharedRcts(model, Met1, Met2):
-    '''
-    This function lists all reactions which are shared by two metabolites
-    
-    Parameters
-    ----------
-    model : cobra model
-        The cobra model for analysis.
-    Met1 : str
-        Metabolite id.
-    Met2 : str
-        Metabolite id.
-
-    Returns
-    -------
-    list, List of shared reactions.
-
-    '''
-    return [reaction for reaction in model.metabolites.get_by_id(Met1).reactions if reaction in model.metabolites.get_by_id(Met2).reactions]
-
-def MetFluxConnect(model, Met1, Met2, SubOff='EX_glc__D_e', ngam=False, slim=True):
-    '''
-    Test whether there exists a flux between two metabolites. The first metabolite is set as source and FBA is optimized to the second metabolite.
-
-    Parameters
-    ----------
-    model : cobra model
-        .
-        The cobra model for analysis.
-    Met1 : str
-        Metabolite id.
-    Met2 : str
-        Metabolite id.
-    SubOff: str
-        Substrate id to be set to zero
-    ngam: str
-        non growth associated maintenance to be set to zero, because checking close metabolites might not fullfill the base ATP demand in ngam
-    slim : Boolean
-        Decision whether only the value of the objective is reported via slim_optimize, or the full flux-reaction list.
-
-    Returns
-    -------
-    list, List of reactions and flux values.
-
-    '''
-    with model:
-        # medium = dict.fromkeys(model.medium, 0)
-        medium = model.medium
-        medium[SubOff] = 0
-        model.medium = medium
-        model.add_boundary(model.metabolites.get_by_id(Met1), type='sink')        
-        model.add_boundary(model.metabolites.get_by_id(Met2), type='sink')
-        if ngam:
-            model.reactions.get_by_id(ngam).lower_bound = 0
-        model.objective = 'SK_{}'.format(Met2)
-        if slim:
-            flux = model.slim_optimize()
-        else:
-            flux = model.optimize()
-            
-        return flux
\ No newline at end of file
