@@ -2,6 +2,76 @@ import ipywidgets as widgets
 import random as rd
 import numpy as np
 
+##########
+##########
+##########
+# (general) function to initiate the homework_notebook Student_ID request
+##########
+##########
+##########
+
+def generate_initiation_interface():
+    # generates a GUI with an 
+    # IntText inut for the STudent_ID,
+    # Button to start iniate_Params_HW(#Homeworknumber)() with
+    # output to print the values from iniate_Params_HW(#Homeworknumber)() in
+    StudID = widgets.IntText( description='Student ID:', value=0 )
+    button_StudID   = widgets.Button( description='Generate Parameters' )
+    output = widgets.Output()
+
+    vb=widgets.VBox([StudID,button_StudID, output])
+    return vb
+
+##########
+##########
+##########
+# functions for compute_Params() for every homework_notebook
+##########
+##########
+##########
+
+def compute_Params_HW3(b):
+    # diable Button from initatl GUI
+    # create hw3 in global namespace
+    # store homework_three instance in hw3
+    # print in output widget of inital GUI
+    vb.children[1].disabled = True
+    global hw3
+    hw3 = homework_three(vb.children[0].value)
+    with vb.children[2]:
+        print("##########\n\nParameters for Question 1:\n\n"\
+        "Your glucose yield Y_(XS) is {} C-mol glucose/C-mol biomass\n\n"\
+        "Your biomass composition is CH_({})O_({})N_({})\n\n##########".format(hw3.newYxs,hw3.newH,hw3.newO,hw3.newN))
+        print("##########\n\nParameters for Question 2:\n\n"\
+        "Your O_2 Exchange-Rate is {} mol/h\n\n"\
+        "Your CO_2 Exchange-Rate is {} mol/h\n\n##########".format(hw3.newRO2,hw3.newRCO2))
+
+#####
+#####
+#####
+
+def compute_Params_HW4(b):
+    # diable Button from initatl GUI
+    # create hw4 in global namespace
+    # store homework_three instance in hw3
+    # print in output widget of inital GUI
+    vb.children[1].disabled = True
+    global hw4
+    hw4 = homework_four(vb.children[0].value)
+    with vb.children[2]:
+        #NOTE: Chemical Compositions are, as of yet, not actually modified in HW4 
+        #      but are treated like it for the sake of the students
+        print("##########\n\nParameters for Question 1:\n\n"\
+        "Approximate Chemical Compostion of Fats: CH_(1.92)O_(0.12)\n" \
+        "Approximate Chemical Compostion of Proteins: CH_(1.57)O_(0.32)N_(0.26)\n\n" \
+        "Heat of combustion for protein: {} kcal/g,\n" \
+        "Heat of combustion for Carbohydrates: {} kcal/g,\n" \
+        "Heat of combustion for Fat: {} kcal/g\n\n##########".format(hw4.newCoP,hw4.newCoC,hw4.newCoF))
+        print("##########\n\nParameters for Question 3:\n\n"\
+        "Oxygen consumption rate: {} mol/h,\n" \
+        "Carbon dioxide production rate: {} mol/h,\n" \
+        "Nitrogen secretion rate: {} g N/h\n\n##########".format(hw4.newOCR,hw4.newCPR, hw4.newNSR))
+        
 ##########
 ##########
 ##########
@@ -468,7 +538,7 @@ class homework_three():
             "Don't forget, that for Question 2, Ethanol is an additional product!\nYou also need to subtract the ash from the assumed Biomass.\n\n"\
             "* First, convert the biomass concentration from g/L to c-mol\n" \
             "  and normalize the gas exchange rates\n\n" \
-            "* You can use the growth rates and exchange rates t determine\n  the yields of O_2 and CO_2\n\n" \
+            "* You can use the growth rates and exchange rates to determine\n the yields of O_2 and CO_2\n\n" \
             "* How to solve the matrix can be seen in 'Example 1' (p. 19/20 of Lecture 4)\n"
             "##########\n"
             print(txt)
@@ -626,17 +696,18 @@ class homework_four():
                 mf = 12+1.92+16*0.12 #CH_1.92O_0.12
             
                 #Check if student solutions are correct
+                counter = 0
                 #Assumes aswer was rounded to 2 decimals
                 if round(mp*self.newCoP,2) != YxqP:
-                    print("Wrong CoP\nDon't give up and try again!")
-                    self.question_one_display.children[-1].clear_output(wait=True)
-                elif round(mc*self.newCoC,2) != YxqC:
-                    print("Wrong CoC\nDon't give up and try again!")
-                    self.question_one_display.children[-1].clear_output(wait=True)
-                elif round(mf*self.newCoF,2) != YxqF:
-                    print("Wrong CoF\nDon't give up and try again!")
-                    self.question_one_display.children[-1].clear_output(wait=True)
-                elif round(mp*self.newCoP,2) == YxqP & round(mc*self.newCoC,2) == YxqC & round(mf*self.newCoF,2) == YxqF:
+                    counter += 1
+                    print("Wrong Combustion of Protein\n")
+                if round(mc*self.newCoC,2) != YxqC:
+                    counter += 1
+                    print("Wrong Combustion of Carbohydrates\n")
+                if round(mf*self.newCoF,2) != YxqF:
+                    counter += 1
+                    print("Wrong Combustion of Fat\n")
+                if round(mp*self.newCoP,2) == YxqP and round(mc*self.newCoC,2) == YxqC and round(mf*self.newCoF,2) == YxqF:
                     self.question_one_display.children[-3].children[0].disabled = True
                     self.question_one_display.children[-3].children[1].disabled = True
                     print("Solution corect!")
@@ -658,6 +729,9 @@ class homework_four():
                     # PRINT THE CODE
                     print("Here's your code!\n\n"+solcode+"\n\nPlease upload it in the designated Moodle-Task.") 
 
+                if counter != 0:
+                    print("\n##########\n Don't give up and try again!\n\n")
+                    self.question_one_display.children[-1].clear_output(wait=True)
 
 
 ###############
@@ -707,20 +781,23 @@ class homework_four():
             with self.question_two_display.children[-1]:
                 counter = 0
                 
+                print("Y_cs: {1} + {2} * Y_co + {3} * Y_cn\n-----------------------------\n")
                 for i in range(len(student_answer_Ycs)):
                     if round(student_answer_Ycs[i],1) != sol_Ycs[i]:
                         counter += 1
-                        print('{}. variable of Ycs function is WRONG\n'.format(i))
+                        print('Variable {} is WRONG\n'.format(i+1))
+                print("Y_cf: {1} + {2} * Y_co + {3} * Y_cn\n-----------------------------\n")
                 for i in range(len(student_answer_Ycf)):
                     if round(student_answer_Ycf[i],1) != sol_Ycf[i]:
                         counter += 1
-                        print('{}. variable of Ycf function is WRONG\n'.format(i))
+                        print('Variable {} is WRONG\n'.format(i+1))
+                print("Y_cp: {1} + {2} * Y_co + {3} * Y_cn\n-----------------------------\n")
                 for i in range(len(student_answer_Ycp)):
                     if round(student_answer_Ycp[i],1) != sol_Ycp[i]:
                         counter += 1
-                        print('{}. variable of Ycp function is WRONG\n'.format(i))
+                        print('Variable {} is WRONG\n'.format(i+1))
                 if counter != 0:
-                    print('\n##########\nPLEASE TRY AGAIN!\n\n')
+                    print("\n##########\n Don't give up and try again!\n\n")
                     self.question_two_display.children[-1].clear_output(wait=True)
                 elif counter == 0:
                     self.question_two_display.children[-3].children[0].disabled = True
@@ -838,7 +915,7 @@ class homework_four():
                 
                 
                 if counter != 0:
-                    print('\n##########\nPLEASE TRY AGAIN!\n\n')
+                    print("\n##########\n Don't give up and try again!\n\n")
                     self.question_three_display.children[-1].clear_output(wait=True)
                 elif counter == 0:
                     self.question_three_display.children[-3].children[0].disabled = True
